type Macros = any;
type RenderContext = Record<string, any>;
type AST_TYPE = 'else' | 'end' | 'foreach' | 'define' | 'content' | 'comment' | 'macro' | 'macro_call' | 'method' | 'index' | 'property' | 'break' | 'noescape' | 'macro_body' | 'raw';
interface CommonAstType {
    value: any;
    id: string;
    pos: {
        first_line: number;
        first_column: number;
    };
}
interface MacroAST extends CommonAstType {
    type: 'macro';
    id: string;
    args: Param[];
}
type SetAST = CommonAstType & {
    type: 'set';
    equal: [ReferencesAST, VELOCITY_AST];
};
interface StringAST extends CommonAstType {
    type: 'string';
    value: string;
    isEval: boolean;
}
type ArrayAST = NormalArrayAST | RangeArrayAST;
interface NormalArrayAST extends CommonAstType {
    type: 'array';
    value: Param[];
}
interface RangeArrayAST extends CommonAstType {
    type: 'array';
    value: [ReferencesAST | string | number];
    isRange: true;
}
interface IfAST extends CommonAstType {
    type: 'if' | 'elseif';
    condition: VELOCITY_AST;
}
type Attribute = Method | IndexAttribute | {
    type: 'property';
    id: string;
};
type IndexAttribute = {
    type: 'index';
    id: LiteralAST | ReferencesAST;
};
interface Method {
    type: 'method';
    id: string;
    args?: Param[];
}
type Param = ReferencesAST | LiteralAST;
type LiteralAST = ArrayAST | {
    type: 'map';
    value: Record<string, Param>;
} | StringAST | {
    type: 'integer' | 'decimal';
    value: string;
} | {
    type: 'bool';
    value: 'true' | 'false' | 'null';
} | {
    type: 'runt';
    value: string;
};
interface ReferencesAST extends CommonAstType {
    type: 'references';
    prue: boolean;
    isWraped: boolean;
    id: string;
    leader: string;
    path?: Attribute[];
    args?: Param[];
}
interface MacroCallAST extends CommonAstType {
    type: 'macro_call';
    args: Param[];
}
interface EachAST extends CommonAstType {
    type: 'foreach';
    from: ReferencesAST;
    to: string;
}
interface MathAST extends CommonAstType {
    type: 'math';
    expression: Array<MacroAST | LiteralAST | ReferencesAST>;
    operator: string;
}
interface CommonAST extends CommonAstType {
    type: AST_TYPE;
}
type VELOCITY_AST = CommonAST | ReferencesAST | MacroCallAST | EachAST | IfAST | MacroAST | MathAST | SetAST | LiteralAST;
interface CompileConfig {
    escape?: boolean;
    unescape?: Record<string, boolean>;
    valueMapper?: (value: any) => any;
    customMethodHandlers?: Array<{
        uid: string;
        match: (payload: {
            property: string;
            context: any;
            params: any[];
        }) => boolean;
        resolve: (payload: {
            property: string;
            context: any;
            params: any[];
        }) => any;
    }>;
    env?: string;
}

declare class Velocity {
    context: RenderContext;
    protected macros: Record<string, Function>;
    protected defines: Record<string, VELOCITY_AST[]>;
    protected conditions: string[];
    protected local: Record<string, unknown>;
    protected silence: boolean;
    protected unescape: {};
    protected contextId: string;
    protected config: CompileConfig;
    protected macrosStore: Record<string, {
        asts: VELOCITY_AST[];
        args: Param[];
    }>;
    protected asts: VELOCITY_AST[];
    protected runState: {
        stop: boolean;
        break: boolean;
    };
    constructor(asts: VELOCITY_AST[], config?: CompileConfig);
    render(_context?: RenderContext, _macros?: Macros, _silence?: boolean): string;
    protected getReferences(_ast: VELOCITY_AST, _isVal?: boolean): string;
    protected getLiteral(_ast: VELOCITY_AST): string;
    protected getExpression(_ast: VELOCITY_AST): string;
    protected setValue(_ast: SetAST): void;
    protected getBlock(_ast: VELOCITY_AST[]): string;
    protected getMacro(_ast: VELOCITY_AST): string;
    protected evalStr(_str: string): string;
    protected eval(str: string, _local?: object): string;
}

type BlockConfig = Record<string, boolean>;
/**
 * @param str string to parse
 * @param blocks self define blocks, such as `#cms(1) hello #end`
 * @param ignoreSpace if set true, then ignore the newline trim.
 */
declare const parse: (str: string, blocks?: BlockConfig, ignoreSpace?: boolean) => VELOCITY_AST[];

/**
 * 获取引用文本，当引用自身不存在的情况下，需要返回原来的模板字符串
 * get variable text
 */
declare function getRefText(ast: ReferencesAST | MacroCallAST): string;

declare const render: (template: string, context?: RenderContext, macros?: Macros, config?: CompileConfig) => string;
declare const Helper: {
    getRefText: typeof getRefText;
};

export { Velocity as Compile, Helper, parse, render };
